<?php

namespace App\Http\Controllers;

use Illuminate\Http\UploadedFile;

class FileGestionController extends Controller
{
    /**
     * Check if there is an error while upload the file
     * 
     * @param array $file An array with infos about the file (from $_FILES)
     * 
     * @return boll If no error
     */
    function is_uploaded_correctly($file)
    {
        $phpFileUploadErrors = array(
            0 => 'There is no error, the file uploaded with success',
            1 => 'The uploaded file exceeds the upload_max_filesize directive in php.ini',
            2 => 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form',
            3 => 'The uploaded file was only partially uploaded',
            4 => 'No file was uploaded',
            6 => 'Missing a temporary folder',
            7 => 'Failed to write file to disk.',
            8 => 'A PHP extension stopped the file upload.',
        );
        echo "<p>" . $phpFileUploadErrors[$file];
        return ($file['error'] == 0);
    }

    /**
     * Script that upload a pdf, png or jpeg file
     * 
     * @param array $file An array with infos about the file (from $_FILES)
     * 
     * @return string|false the path to the file in the server or false if something went wrong
     */
    function upload_file($file)
    {

        //Get the file name
        $file_name = $file['name'];

        //Get the file extention
        $file_extention = explode('.', $file_name)[1];

        //Check that the extention is correct
        switch ($file_extention) {
            case "pdf":
            case "jpeg":
            case "jpg":
            case "png":
                echo "<p>Upload the " . $file_name . " file!</p>";
                break;
            default:
                echo "<p>The file extention should be a .pdf, .jpeg or .png, not a ." . $file_extention . " extention!</p>";
                return false;
        }

        $upload_path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/" . $file_name;

        if (!move_uploaded_file($file['tmp_name'], $upload_path)) {
            echo "<p>Cannot upload the " . $file_name . " file!</p>";
            return false;
        }
        return $upload_path;
    }

    /**
     * Script that encrypt a file
     * 
     * @param array $file An array with infos about the file (from $_FILES)
     * 
     * @return array the path to the encrypted file and the initialisation vector
     */
    function file_encrypt(UploadedFile $file)
    {
        //Get the file name
        $file_name = $file->getClientOriginalName();

        //Get the file extention
        $file_extention = explode('.', $file_name)[1];

        //Encryption key
        $encryption_key = md5('jed4n j4k0 v3l1k1 kljuc');
        //Convert the file into code
        $data = file_get_contents($file->getPathName());
        //Select cipher method AES
        $cipher = "AES-128-CTR";

        //Create IV with correct length
        $iv_length = openssl_cipher_iv_length($cipher);

        // Non-NULL initialization vector for encryption 
        //Random length 16 bytes
        $encryption_iv = random_bytes($iv_length);

        // Encrypt data with openssl 
        $data = openssl_encrypt($data, $cipher, $encryption_key, 0, $encryption_iv);

        //Save data
        $encrypted_file = base64_encode($data);

        //Store the encrypted file
        $upload_path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/encrypted_" . $file_name;
        file_put_contents($upload_path, $encrypted_file);

        return array($upload_path, $encryption_iv);
    }

    /**
     * Script that decrypt a file
     * 
     * @param array $encrypted_info the path to the encrypted file and the initialisation vector
     * 
     * @return string the path to the file
     */
    function file_decrypt($encrypted_info)
    {

        $file = $encrypted_info[0];

        $path_array = explode("/", $file);
        $file_name = $path_array[array_key_last($path_array)];

        //Get the encrypted code.
        $encrypted_code = file_get_contents($file);

        //Create the key
        $decryption_key = md5('jed4n j4k0 v3l1k1 kljuc');

        //Select cipher method AES
        $cipher = "AES-128-CTR";

        // Non-NULL initialization vector for encryption 
        $decryption_iv = $encrypted_info[1];
        $data = base64_decode($encrypted_code);

        // Decrypt data:
        $data = openssl_decrypt($data, $cipher, $decryption_key, 0, $decryption_iv);

        $tmp_upload_path = sys_get_temp_dir() . $file_name;
        //Save the decrypted code somewhere.
        file_put_contents($tmp_upload_path, $data);

        return $tmp_upload_path;
    }

    /**
     * Delete a file from the server
     * 
     * @param string $file the name of the file to be deleted
     * 
     * @return bool if nothing wrong happend
     */
    function delete_file($file)
    {
        $path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/" . $file;

        // Check if the file exist
        if (!file_exists($path)) {
            echo "<p>Cannot find the file " . $path . "</p>";
            return false;
        }

        if (!($encrypted_info_file = fopen(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt", "a+"))) {
            echo "<p>Unable to open the data of the file</p>";
            return false;
        }

        //Read the infos
        $infos = fread($encrypted_info_file, filesize(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt"));

        // Close the file
        fclose($encrypted_info_file);

        // Let's find the matching line
        $pattern = "/" . str_replace('/', "\/", $path) . ",(.*)/";
        if (!(preg_match_all($pattern, $infos, $matches))) {
            echo "<p>Unable to find the file's data</p>";
            return false;
        }

        // Delete the line
        $contents = str_replace($matches[0][0] . "\n", '', $infos);

        //Reopen the file to rewite it
        $encrypted_info_file = fopen(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']) . "/../resources/files/eif.txt", "w+");

        // Rewrite the file
        fwrite($encrypted_info_file, $contents);

        // Close the file
        fclose($encrypted_info_file);

        // Delete the file
        unlink($path);

        return true;
    }
}
